/*
 * Nautilus floating bar used for cursor status in editor.
 */

/* Floating status bar */
.floating-bar {
  padding: 2px;
  background-color: @theme_base_color;
  border-width: 1px;
  border-style: solid solid none;
  border-color: @borders;
  border-radius: 3px 3px 0 0;
}

.floating-bar.bottom.left { /* axes left border and border radius */
  border-left-style: none;
  border-top-left-radius: 0;
}
.floating-bar.bottom.right { /* axes right border and border radius */
  border-right-style: none;
  border-top-right-radius: 0;
}

.floating-bar:backdrop {
  background-color: @theme_unfocused_base_color;
  border-color: @unfocused_borders;
}

.floating-bar .button {
    padding: 4px;
}



/*
 * Search entry that slides down from top of editor.
 */
GbEditorFrame .gb-search-slider {
    background-color: @theme_base_color;
    padding: 6px;
    border-color: shade (@theme_base_color, 0.60);
    border-radius: 0 0 5px 5px;
    border-width: 0 1px 1px 1px;
    border-style: solid;
}
.gb-search-entry-occurrences-tag {
    color: shade (@theme_unfocused_fg_color, 0.8);
    margin: 2px;
    padding: 2px;
}

GdTaggedEntry {
    color: @theme_fg_color;
}


/*
 * Workspace switcher, thin-panel on left side.
 */
GtkStackSwitcher.gb-workspace-switcher > GtkRadioButton {
    background-color: transparent;
    background-image: none;
    border-radius: 4px 4px 4px 4px;
    border-width: 1px 1px 1px 1px;
    border-style: none;
    box-shadow: none;
    -GtkButton-image-spacing: 0;
    -GtkButton-inner-border: 0;
    padding: 4px 1px 4px 1px;
}
GtkStackSwitcher.gb-workspace-switcher > GtkRadioButton:checked {
    background-color: shade(@theme_base_color, 0.70);
    color: #ffffff;
}
GtkStackSwitcher.gb-workspace-switcher > GtkRadioButton:checked:hover {
    border-color: shade(@theme_base_color, 0.60);
}
GtkStackSwitcher.gb-workspace-switcher > GtkRadioButton:hover {
    border-style: solid;
    border-width: 1px 1px 1px 1px;
    border-color: shade(@theme_base_color, 0.80);
    transition-duration: 0ms;
}
GtkStackSwitcher.gb-workspace-switcher > GtkRadioButton:active {
    background-color: shade(@theme_base_color, 0.80);
    border-style: solid;
    border-width: 1px 1px 1px 1px;
    border-color: shade(@theme_base_color, 0.60);
    transition-duration: 0ms;
}


/*
 * Preferences sidebar style.
 */
.preferences.sidebar GtkListBox {
    background-color: shade(@theme_bg_color, 0.99);
}
.preferences.sidebar GtkListBoxRow {
    padding: 10px;
}


/**
 * Preference page language style.
 */
GbPreferencesPageLanguage GtkSearchEntry {
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0;
}
GbPreferencesPageLanguage GtkScrolledWindow {
    border-top-width: 0;
}


/*
 * Command Bar styling.
 */
GtkBox.gb-command-bar-box {
    border: none;
    background-image: linear-gradient(to bottom, #2e3436, #555753 10%);
    box-shadow: inset 0px 3px 6px #2e3436;
}

GtkBox.gb-command-bar-box GtkLabel {
    color: #eeeeec;
}

GtkEntry.gb-command-bar-entry {
    font-family: Monospace;
    background-image: none;
    padding: 6px 6px 6px 6px;
    color: #eeeeec;
}


/*
 * Styling of main window header bar.
 */
GbWorkbench GtkHeaderBar {
    border-bottom: none;
    box-shadow: none;
}


/*
 * Tab header styling.
 */
GtkMenuButton.tab-header-first .button,
GtkButton.tab-header-first {
    border-left: none;
    border-top-left-radius: 0px;
    border-bottom-left-radius: 0px;
    box-shadow: none;
}
GtkMenuButton.tab-header-last .button,
GtkButton.tab-header-last {
    border-right: none;
    border-top-right-radius: 0px;
    border-bottom-right-radius: 0px;
    box-shadow: none;
}
GtkMenuButton.tab-control-first .button,
GtkButton.tab-control-first {
    border-left: none;
    border-top-left-radius: 0px;
    border-bottom-left-radius: 0px;
    box-shadow: none;
}
GtkMenuButton.tab-control-last .button,
GtkButton.tab-control-last {
    border-right: none;
    border-top-right-radius: 0px;
    border-bottom-right-radius: 0px;
    box-shadow: none;
}



#project-title {
    border-left: none;
    border-right: none;
    border-top-left-radius: 0px;
    border-bottom-left-radius: 0px;
    border-top-right-radius: 0px;
    border-bottom-right-radius: 0px;
}


GbCreditsWidget GtkEventBox {
    background-color: #000000;
}

GbCreditsWidget GtkLabel {
    color: #eeeeef;
}


GbSourceStyleSchemeWidget GtkSourceView {
    font-family: Monospace;
}


GtkScrolledWindow.gb-linked-scroller {
    border-top: none;
}


/*
 * Disable various animatinos that are enabled by default and really annoying
 * to the overall flow of Builder.
 */
GbSearchDisplayGroup GtkListBox .list-row {
    transition: none;
}
GbDocumentStack .button {
    transition: none;
}
